from fastapi import APIRouter, Depends

from id_broker_common.kelvin import kelvin_session
from self_disclosure_plugin.dependencies import (
    extract_members,
    get_single_user_by_broker_id,
    schoolclass_getter,
    token_dep,
)
from self_disclosure_plugin.models import User, UsersClasses
from ucsschool.apis.models import OAuth2Token
from ucsschool.apis.opa import OPAClient, opa_instance
from ucsschool.kelvin.client import SchoolClass, Session, User as KelvinUser

router = APIRouter(tags=["users"])


@router.get("/{user_id}/metadata", response_model=User)
async def user_metadata(
    user_id: str,
    token: OAuth2Token = Depends(token_dep),
    policy_instance: OPAClient = Depends(opa_instance),
    session: Session = Depends(kelvin_session),
):
    await policy_instance.check_policy_true_or_raise(
        "self_disclosure_plugin/users_self_info", {"sub": token.sub, "user_id": user_id}
    )
    user = await get_single_user_by_broker_id(user_id, session)

    return User(
        user_id=user_id,
        username=user.name,
        firstname=user.firstname,
        lastname=user.lastname,
        type=", ".join(user.roles),  # TODO what exactly to use here?
        school_id=user.school,
    )


@router.get("/{user_id}/groups", response_model=UsersClasses)
async def user_groups(
    user_id: str,
    token: OAuth2Token = Depends(token_dep),
    policy_instance: OPAClient = Depends(opa_instance),
    session: Session = Depends(kelvin_session),
):
    await policy_instance.check_policy_true_or_raise(
        "self_disclosure_plugin/users_self_info", {"sub": token.sub, "user_id": user_id}
    )
    user: KelvinUser = await get_single_user_by_broker_id(user_id, session)

    groups = []
    for school, classes in user.school_classes.items():
        for classname in classes:
            name = f"{school}-{classname}"
            # get student count, which means students only!
            school_class: SchoolClass = await schoolclass_getter(school, classname, session)
            classmembers = await extract_members(school_class, session)
            groups.append(dict(group_id=name, name=name, student_count=len(classmembers.students)))

    return UsersClasses(groups=groups)
